==> kom%2B%2B-0.1.tar.gz: kom++-0.1/other-clients/test2.cc <==
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>

#include "session.h"		// High-level objects
#include "conference.h"
#include "text.h"
#include "person.h"
#include "maps.h"

#include "array.h"
#include "connection.h"
#include "obsolete.h"
#include "uplink.h"
#include "who-info.h"
#include "membership.h"
#include "read-order.h"
#include "unread-confs.h"
#include "text-mapping.h"
#include "server-info.h"

#include "get_membership_q.h"	// Questions (from libquestion.a)
#include "get_unread_confs_q.h"
#include "get_time_q.h"


Connection   * port = NULL;
Session        session;

Unread_confs * unread_confs;

Conf_no        current_conf = 0;
Pers_no        this_person;


static void
find_password(char *res)
{
    char buffer[100];
    FILE *fp = fopen(sprintf(buffer, "/users/%s/.kompwd", getenv("USER")),
		     "r");
    assert(fp != NULL);
    fgets(res, 20, fp);
    *strchr(res, '\n') = '\0';
}


// ================================================================
//                     Implementation of commands


void
map_command(prot_a_LyStr &str)
{
    Conf_no         cno;
    Local_text_no   lno;
    Text_no         tno;

    cno = str.stol();
    lno = str.stol();
    cout << "cno == " << cno << " och lno == " << lno << endl;
    if (cno == 0 || lno == 0)
	return;

    tno = session.maps()[cno][lno];
    cout << "Text nummer " << lno << " i m|te " << cno << " {r "
	 << tno << ".\n";
}


void
print_text(Text ts)
{
    int       i;

    Conferences &  conferences = session.conferences();
    Texts &        texts = session.texts();

    cout << ts.text_no() << " /" << ts.length() << " tecken/ " << ' '
	 << conferences[ts.author()].name() << '\n';

    for (i = 0; i < ts.num_uplinks(); ++i) {
	Text_no   parent;

	parent = ts.uplink(i)->parent();
	cout << "Kommentar till text " << parent << " av "
	     << conferences[texts[parent].author()].name() << '\n';
    }

    for (i = 0; i < ts.num_recipients(); ++i) {
	cout << "Mottagare: "
	     << conferences[ts.recipient(i)->recipient()].name() 
	     << " <" << ts.recipient(i)->local_no() << ">\n";
    }

    cout << "[rende...\n";
    cout << "------------------------------------------------------------\n";
    cout << ts.text_mass() << endl;
    cout << '(' << ts.text_no() << ") -----------------------------------\n";
    for (i = 0; i < ts.num_footnote_in(); ++i) {
	Text_no   footnote;

	footnote = ts.footnote_in(i);
	cout << "Fotnot i text " << footnote << " av "
	     << conferences[texts[footnote].author()].name() 
	     << '\n';
    }

    for (i = 0; i < ts.num_comment_in(); ++i) {
	Text_no   comment;

	comment = ts.comment_in(i);
	cout << "Kommentar i text " << comment << " av "
	     << conferences[texts[comment].author()].name() 
	     << '\n';
    }
}


void
print_text_command(prot_a_LyStr &str)
{
    Texts &   texts = session.texts();
    Text      ts;
    Text_no   tn;


    tn = str.stol();
    if (tn != 0) {
	ts = texts[tn];
	if (ts.valid() != TRUE)
	    cout << "Text " << tn << " finns inte.\n";
	else
	    print_text(ts);
    }
}



//
// List the number of unread texts in each conference.
// This code does not take into account deleted texts (yet).
//

void
list_unread_command(prot_a_LyStr &str)
{
    Conferences &              conferences = session.conferences();
    get_unread_confs_question  ucq(port, this_person);
    Array<Conf_no>             unread_confs;
    int                        i;

    if (ucq.receive() != st_ok) {
	cerr << "Unread confs error " << ucq.error() << "\n";
	exit(1);
    }

    cout << "Du har ungef{r s} h{r m}nga ol{sta inl{gg i f|ljande m|ten:\n";
    unread_confs = ucq.result();

    for (i = 0; i < unread_confs.size(); ++i) {
	get_membership_question  * gmq;

	gmq = new get_membership_question(port, this_person, unread_confs[i]);
	if (gmq->receive() == st_ok) {
	    Conference      conf = conferences[unread_confs[i]];
	    Membership      membership = gmq->result();
	    unsigned long   num_unread = (conf.no_of_texts() 
					  + conf.first_local_no()
					  - membership.last_text_read() - 1);

	    if (num_unread > 0) {
		cout << num_unread << '\t' << conf.name() << '\n';
	    }
	}
	delete gmq;
    }
}


// ----------------------------------------------------------------


void
lookup_name_command(prot_a_LyStr &str)
{
    Conferences &       conferences = session.conferences();
    Array<Micro_conf>   matches;
    int                 i;
    
    matches = session.lookup_name(str);
    if (matches.size() == 0) 
	cout << "Inga namn matchade " << str << endl;
    else {
	cout << "F|ljande namn matchar " << str << ":\n";
	for (i = 0; i < matches.size(); ++i) {
	    cout << matches[i].confno() << '\t' 
		 << conferences[matches[i].confno()].name() 
		 << endl;
	}
    }
    cout << endl;
}


// ----------------------------------------------------------------


void
lookup_persons_command(prot_a_LyStr &str)
{
    Conferences &    conferences = session.conferences();
    Array<Pers_no>   persons;
    int              i;
    
    persons = session.regexp_lookup_persons(str);
    if (persons.size() == 0) 
	cout << "Inga personer matchade " << str << endl;
    else {
	cout << "F|ljande personer matchar " << str << ":\n";
	for (i = 0; i < persons.size(); ++i) {
	    cout << persons[i] << '\t' << conferences[persons[i]].name() 
		 << endl;
	}
    }
    cout << endl;
}


// ----------------------------------------------------------------


void
next_text_command(prot_a_LyStr &str)
{
    Texts &   texts = session.texts();
    Text      ts;
    Text_no   tn;

    if (unread_confs->next_prompt() == NO_MORE_TEXT) {
	cout << "Det finns ingen mer text\n";
    } else {
	tn = unread_confs->next_text(TRUE);
	ts = texts[tn];
	if (ts.valid() != TRUE)
	    cout << "Text " << tn << " finns inte.\n";
	else
	    print_text(ts);
    }
}


// ----------------------------------------------------------------


void
next_conf_command(prot_a_LyStr &str)
{
    current_conf = unread_confs->goto_next_conf();

    if (current_conf == 0) {
	cout << "Inga fler olsta\n";
    } else {
	cout << session.conferences()[current_conf].name()
	     << " (" << unread_confs->min_unread_texts_in_current() 
	     << " ol{sta)\n";
    }
}


// ----------------------------------------------------------------


void
who_is_on_command(prot_a_LyStr &str)
{
    Conferences &        conferences = session.conferences();
    Array<Who_info>      who_info_list;
    who_is_on_question   wioq(port);
    int                  i;
    
    if (wioq.receive() == st_error)
	cerr << "Kunde inte h{mta vilkalistan.  :-(\n";
    else {
	who_info_list = wioq.result();
	for (i = 0; i < who_info_list.size(); ++i) {
	    Conf_no wc = who_info_list[i].working_conf();
	    
	    if (wc != 0)
		conferences[wc].prefetch();
	    
	    conferences[who_info_list[i].pers_no()].prefetch();
	}
	for (i = who_info_list.size() - 1; i >= 0 ; --i) {
	    Conf_no wc = who_info_list[i].working_conf();
	    
	    cout << who_info_list[i].session() << '\t'
		 << conferences[who_info_list[i].pers_no()].name()
		 << "\n\t";
	    if (wc != 0)
		cout << conferences[wc].name();
	    else
		cout << "ej n{rvarande i n}got m|te";
	    cout << '\n';
	}
    }
}


// ================================================================
//                       The command interpreter


void   list_commands_command(prot_a_LyStr &str);

struct command_struct {
    char    shortcommand[5];
    char  * description;
    void (* function)(prot_a_LyStr &);
} commands[] = {
    {"mpMP", "Map",                  map_command},
    {"lkLK", "lista kommandon",      list_commands_command},
    {"laLA", "lookup nAmn",          lookup_name_command},
    {"lnLN", "lista nyheter",        list_unread_command},
    {"lpLP", "lookup persons",       lookup_persons_command},
    {"niNI", "n{sta inl{gg",         next_text_command},
    {"nmNM", "n{sta m|te",           next_conf_command},
    {"slSL", "sluta",                NULL},
    {"viVI", "vilka",                who_is_on_command},
    {"}t]T", "}terse text <nummer>", print_text_command},
};

int num_commands = sizeof(commands) / sizeof(command_struct);


void
list_commands_command(prot_a_LyStr &str)
{
    int   i;
    
    for (i = 0; i < num_commands; ++i) {
	cout << commands[i].shortcommand[0]
	     << commands[i].shortcommand[1] << '\t'
	     << commands[i].description << '\n';
    }
}


void
command_interpreter()
{
    char    ch;
    char    buf[2];
    prot_a_LyStr   command_line;

    while (1) {
	Prompt   prompt;
	char   * promptstr;

	command_line.clear();
	while ((prompt = unread_confs->next_prompt()) == TIMEOUT_PROMPT)
	{
	    printf(">");
	    fflush(stdout);
	}
	    
	if (current_conf == 0 && prompt == NEXT_TEXT)
	    prompt = NEXT_CONF;

	switch (prompt) {
	  case NEXT_CONF:
	    promptstr = "G} till n{sta m|te> "; 
	    break;
	  case NEXT_TEXT:
	    promptstr = "(L{sa) n{sta text> ";
	    break;
	  case NEXT_COMMENT:
	    promptstr = "(L{sa) n{sta kommentar> ";
	    break;
	  case NEXT_FOOTNOTE:
	    promptstr = "(L{sa) n{sta fotnot> ";
	    break;
	  case NO_MORE_TEXT:
	    promptstr = "(Se) tiden> ";
	    break;
	  case TIMEOUT_PROMPT:
	    abort();
	  case PROMPT_ERROR:
	    abort();
	}
	cout << promptstr;

	// Read the command from stdin
	while ((ch = cin.get()) != '\n')
	    command_line << ch;
	command_line.skipspace();

	if (command_line.strlen() == 0) {

	    // Default command.  Depends on what the prompt was.
	    switch (prompt) {
	      case NEXT_CONF:
		next_conf_command(command_line);
		break;
	      case NEXT_TEXT:
	      case NEXT_COMMENT:
	      case NEXT_FOOTNOTE:
		next_text_command(command_line);
		break;
	      case NO_MORE_TEXT:
		{
		    get_time_question q(port);
		    if (q.receive() == st_ok) {
			struct tm t = q.result();
			cout << "Klockan {r " << 1900+t.tm_year 
			    << '-' << setw(2) << setfill('0') << 1+t.tm_mon
			    << '-' << setw(2) << setfill('0') << t.tm_mday
			    << ' ' << setw(2) << setfill('0') << t.tm_hour
			    << ':' << setw(2) << setfill('0') << t.tm_min
			    << ':' << setw(2) << setfill('0') << t.tm_sec
			    << " enligt servern." << endl;
		    } else {
			cout << "Klockan {r inte." << endl;
		    }
		}
		break;
	      case TIMEOUT_PROMPT:
		abort();
	      case PROMPT_ERROR:
		abort();
	    }
	    continue;
	    
	} else if (command_line.strlen() == 1) {
	    cout << "Ok{nt kommando\n";
	    continue;

	} else {
	    buf[0] = command_line[0];
	    buf[1] = command_line[1];
	    command_line.remove_first(2);
	    command_line.skipspace();
	}

	for (int i = 0; i < num_commands; ++i) {
	    if ( ( (buf[0] == commands[i].shortcommand[0])
		   || (buf[0] == commands[i].shortcommand[2]))
		 && ( (buf[1] == commands[i].shortcommand[1])
		      || (buf[1] == commands[i].shortcommand[3])))
	    {
		if (commands[i].function == NULL)
		    return;
		else {
		    commands[i].function(command_line);
		    break;
		}
	    }
	}
	if (i == num_commands)
	    cout << "Ok{nt kommando!\n";
    }
}


// ================================================================


int main(int argc,
	 char **argv)
{
    char pwd[100];
    char *server_name = "kom.lysator.liu.se";

    this_person = argc >= 2 ? atol(argv[1]) : 19;
    find_password(&pwd[0]);

    if (argc >= 3)
	server_name = argv[2];

    port = new Connection(server_name, "4894");
    session.set_connection(port);

    {
	Server_info  * sinfo;
	Text_no        motd;

	sinfo = port->get_server_info();
	motd = sinfo->get_motd_of_lyskom();
	cout << "motd_of_lyskom = " << motd << endl;

	cout << "Message of the day:\n";
	print_text(session.texts()[motd]);
    }

    if (session.login(this_person, &pwd[0]) != st_ok)
    {
	cerr << "Login error.\n";
	exit(1);
    }

    unread_confs = new Unread_Confs(&session, this_person);
    command_interpreter();
}

==> kom%2B%2B-0.2.tar.gz: kom++-0.2/other-clients/test2.cc <==
// Simple proof-of-concept LysKOM client using kom++.
//
// $Id: test2.cc,v 1.41 1994/07/16 17:48:37 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>

#include "session.h"		// High-level objects
#include "conference.h"
#include "text.h"
#include "person.h"
#include "maps.h"

#include "array.h"
#include "connection.h"
#include "obsolete.h"
#include "uplink.h"
#include "who-info.h"
#include "membership.h"
#include "read-order.h"
#include "unread-confs.h"
#include "text-mapping.h"
#include "server-info.h"

#include "get_membership_q.h"	// Questions (from libquestion.a)
#include "get_unread_confs_q.h"
#include "get_time_q.h"


Connection   * port = NULL;
Session        session;

Unread_confs * unread_confs;

Conf_no        current_conf = 0;
Pers_no        this_person;


static void
find_password(char *res)
{
    char buffer[100];
    FILE *fp = fopen(sprintf(buffer, "/users/%s/.kompwd", getenv("USER")),
		     "r");
    assert(fp != NULL);
    fgets(res, 20, fp);
    *strchr(res, '\n') = '\0';
}


// ================================================================
//                     Implementation of commands


void
map_command(prot_a_LyStr &str)
{
    Conf_no         cno;
    Local_text_no   lno;
    Text_no         tno;

    cno = str.stol();
    lno = str.stol();
    cout << "cno == " << cno << " och lno == " << lno << endl;
    if (cno == 0 || lno == 0)
	return;

    tno = session.maps()[cno][lno];
    cout << "Text nummer " << lno << " i m|te " << cno << " {r "
	 << tno << ".\n";
}


void
print_text(Text ts)
{
    int       i;

    Conferences &  conferences = session.conferences();
    Texts &        texts = session.texts();

    cout << ts.text_no() << " /" << ts.num_lines() << " rader/ " << ' '
	 << conferences[ts.author()].name() << '\n';

    for (i = 0; i < ts.num_uplinks(); ++i) {
	Text_no   parent;

	parent = ts.uplink(i)->parent();
	cout << "Kommentar till text " << parent << " av "
	     << conferences[texts[parent].author()].name() << '\n';
    }

    for (i = 0; i < ts.num_recipients(); ++i) {
	cout << "Mottagare: "
	     << conferences[ts.recipient(i)->recipient()].name() 
	     << " <" << ts.recipient(i)->local_no() << ">\n";
    }

    cout << "[rende...\n";
    cout << "------------------------------------------------------------\n";
    cout << ts.text_mass() << endl;
    cout << '(' << ts.text_no() << ") -----------------------------------\n";
    for (i = 0; i < ts.num_footnote_in(); ++i) {
	Text_no   footnote;

	footnote = ts.footnote_in(i);
	cout << "Fotnot i text " << footnote << " av "
	     << conferences[texts[footnote].author()].name() 
	     << '\n';
    }

    for (i = 0; i < ts.num_comment_in(); ++i) {
	Text_no   comment;

	comment = ts.comment_in(i);
	cout << "Kommentar i text " << comment << " av "
	     << conferences[texts[comment].author()].name() 
	     << '\n';
    }
}


void
print_text_command(prot_a_LyStr &str)
{
    Texts &   texts = session.texts();
    Text      ts;
    Text_no   tn;


    tn = str.stol();
    if (tn != 0) {
	ts = texts[tn];
	if (ts.valid() != TRUE)
	    cout << "Text " << tn << " finns inte.\n";
	else
	    print_text(ts);
    }
}



//
// List the number of unread texts in each conference.
// This code does not take into account deleted texts (yet).
//

void
list_unread_command(prot_a_LyStr &str)
{
    Conferences &              conferences = session.conferences();
    get_unread_confs_question  ucq(port, this_person);
    Array<Conf_no>             unread_confs;
    int                        i;

    if (ucq.receive() != st_ok) {
	cerr << "Unread confs error " << ucq.error() << "\n";
	exit(1);
    }

    cout << "Du har ungef{r s} h{r m}nga ol{sta inl{gg i f|ljande m|ten:\n";
    unread_confs = ucq.result();

    for (i = 0; i < unread_confs.size(); ++i) {
	get_membership_question  * gmq;

	gmq = new get_membership_question(port, this_person, unread_confs[i]);
	if (gmq->receive() == st_ok) {
	    Conference      conf = conferences[unread_confs[i]];
	    Membership      membership = gmq->result();
	    unsigned long   num_unread = (conf.no_of_texts() 
					  + conf.first_local_no()
					  - membership.last_text_read() - 1);

	    if (num_unread > 0) {
		cout << num_unread << '\t' << conf.name() << '\n';
	    }
	}
	delete gmq;
    }
}


// ----------------------------------------------------------------


void
lookup_name_command(prot_a_LyStr &str)
{
    Conferences &       conferences = session.conferences();
    Array<Micro_conf>   matches;
    int                 i;
    
    matches = session.lookup_name(str);
    if (matches.size() == 0) 
	cout << "Inga namn matchade " << str << endl;
    else {
	cout << "F|ljande namn matchar " << str << ":\n";
	for (i = 0; i < matches.size(); ++i) {
	    cout << matches[i].confno() << '\t' 
		 << conferences[matches[i].confno()].name() 
		 << endl;
	}
    }
    cout << endl;
}


// ----------------------------------------------------------------


void
lookup_persons_command(prot_a_LyStr &str)
{
    Conferences &    conferences = session.conferences();
    Array<Pers_no>   persons;
    int              i;
    
    persons = session.regexp_lookup_persons(str);
    if (persons.size() == 0) 
	cout << "Inga personer matchade " << str << endl;
    else {
	cout << "F|ljande personer matchar " << str << ":\n";
	for (i = 0; i < persons.size(); ++i) {
	    cout << persons[i] << '\t' << conferences[persons[i]].name() 
		 << endl;
	}
    }
    cout << endl;
}


// ----------------------------------------------------------------


void
next_text_command(prot_a_LyStr &str)
{
    Texts &   texts = session.texts();
    Text      ts;
    Text_no   tn;

    if (unread_confs->next_prompt() == NO_MORE_TEXT) {
	cout << "Det finns ingen mer text\n";
    } else {
	tn = unread_confs->next_text(TRUE);
	ts = texts[tn];
	if (ts.valid() != TRUE)
	    cout << "Text " << tn << " finns inte.\n";
	else
	    print_text(ts);
    }
}


// ----------------------------------------------------------------


void
next_conf_command(prot_a_LyStr &str)
{
    current_conf = unread_confs->goto_next_conf();

    if (current_conf == 0) {
	cout << "Inga fler olsta\n";
    } else {
	cout << session.conferences()[current_conf].name()
	     << " (" << unread_confs->min_unread_texts_in_current() 
	     << " ol{sta)\n";
    }
}


// ----------------------------------------------------------------


void
who_is_on_command(prot_a_LyStr &str)
{
    Conferences &        conferences = session.conferences();
    Array<Who_info>      who_info_list;
    who_is_on_question   wioq(port);
    int                  i;
    
    if (wioq.receive() == st_error)
	cerr << "Kunde inte h{mta vilkalistan.  :-(\n";
    else {
	who_info_list = wioq.result();
//	who_info_list = session.who_is_on();
	for (i = 0; i < who_info_list.size(); ++i) {
	    Conf_no wc = who_info_list[i].working_conf();
	    
	    if (wc != 0)
		conferences[wc].prefetch();
	    
	    conferences[who_info_list[i].pers_no()].prefetch();
	}
	for (i = who_info_list.size() - 1; i >= 0 ; --i) {
	    Conf_no wc = who_info_list[i].working_conf();
	    
	    cout << who_info_list[i].session() << '\t'
		 << conferences[who_info_list[i].pers_no()].name()
		 << "\n\t";
	    if (wc != 0)
		cout << conferences[wc].name();
	    else
		cout << "ej n{rvarande i n}got m|te";
	    cout << '\n';
	}
    }
}


// ================================================================
//                       The command interpreter


void   list_commands_command(prot_a_LyStr &str);

struct command_struct {
    char    shortcommand[5];
    char  * description;
    void (* function)(prot_a_LyStr &);
} commands[] = {
    {"mpMP", "Map",                  map_command},
    {"lkLK", "lista kommandon",      list_commands_command},
    {"laLA", "lookup nAmn",          lookup_name_command},
    {"lnLN", "lista nyheter",        list_unread_command},
    {"lpLP", "lookup persons",       lookup_persons_command},
    {"niNI", "n{sta inl{gg",         next_text_command},
    {"nmNM", "n{sta m|te",           next_conf_command},
    {"slSL", "sluta",                NULL},
    {"viVI", "vilka",                who_is_on_command},
    {"}t]T", "}terse text <nummer>", print_text_command},
};

int num_commands = sizeof(commands) / sizeof(command_struct);


void
list_commands_command(prot_a_LyStr &str)
{
    int   i;
    
    for (i = 0; i < num_commands; ++i) {
	cout << commands[i].shortcommand[0]
	     << commands[i].shortcommand[1] << '\t'
	     << commands[i].description << '\n';
    }
}


void
command_interpreter()
{
    char    ch;
    char    buf[2];
    prot_a_LyStr   command_line;

    while (1) {
	Prompt   prompt;
	char   * promptstr;

	command_line.clear();
	while ((prompt = unread_confs->next_prompt()) == TIMEOUT_PROMPT)
	{
	    printf(">");
	    fflush(stdout);
	}
	    
	if (current_conf == 0 && prompt == NEXT_TEXT)
	    prompt = NEXT_CONF;

	switch (prompt) {
	  case NEXT_CONF:
	    promptstr = "G} till n{sta m|te> "; 
	    break;
	  case NEXT_TEXT:
	    promptstr = "(L{sa) n{sta text> ";
	    break;
	  case NEXT_COMMENT:
	    promptstr = "(L{sa) n{sta kommentar> ";
	    break;
	  case NEXT_FOOTNOTE:
	    promptstr = "(L{sa) n{sta fotnot> ";
	    break;
	  case NO_MORE_TEXT:
	    promptstr = "(Se) tiden> ";
	    break;
	  case TIMEOUT_PROMPT:
	    abort();
	  case PROMPT_ERROR:
	    abort();
	}
	cout << promptstr;

	// Read the command from stdin
	while ((ch = cin.get()) != '\n')
	    command_line << ch;
	command_line.skipspace();

	if (command_line.strlen() == 0) {

	    // Default command.  Depends on what the prompt was.
	    switch (prompt) {
	      case NEXT_CONF:
		next_conf_command(command_line);
		break;
	      case NEXT_TEXT:
	      case NEXT_COMMENT:
	      case NEXT_FOOTNOTE:
		next_text_command(command_line);
		break;
	      case NO_MORE_TEXT:
		{
		    get_time_question q(port);
		    if (q.receive() == st_ok) {
			struct tm t = q.result();
			cout << "Klockan {r " << 1900+t.tm_year 
			    << '-' << setw(2) << setfill('0') << 1+t.tm_mon
			    << '-' << setw(2) << setfill('0') << t.tm_mday
			    << ' ' << setw(2) << setfill('0') << t.tm_hour
			    << ':' << setw(2) << setfill('0') << t.tm_min
			    << ':' << setw(2) << setfill('0') << t.tm_sec
			    << " enligt servern." << endl;
		    } else {
			cout << "Klockan {r inte." << endl;
		    }
		}
		break;
	      case TIMEOUT_PROMPT:
		abort();
	      case PROMPT_ERROR:
		abort();
	    }
	    continue;
	    
	} else if (command_line.strlen() == 1) {
	    cout << "Ok{nt kommando\n";
	    continue;

	} else {
	    buf[0] = command_line[0];
	    buf[1] = command_line[1];
	    command_line.remove_first(2);
	    command_line.skipspace();
	}

	for (int i = 0; i < num_commands; ++i) {
	    if ( ( (buf[0] == commands[i].shortcommand[0])
		   || (buf[0] == commands[i].shortcommand[2]))
		 && ( (buf[1] == commands[i].shortcommand[1])
		      || (buf[1] == commands[i].shortcommand[3])))
	    {
		if (commands[i].function == NULL)
		    return;
		else {
		    commands[i].function(command_line);
		    break;
		}
	    }
	}
	if (i == num_commands)
	    cout << "Ok{nt kommando!\n";
    }
}


// ================================================================


int main(int argc,
	 char **argv)
{
    char pwd[100];
    char *server_name = "kom.lysator.liu.se";

    this_person = argc >= 2 ? atol(argv[1]) : 19;
    find_password(&pwd[0]);

    if (argc >= 3)
	server_name = argv[2];

    port = new Connection(server_name, "4894");
    session.set_connection(port);

    {
	Server_info  * sinfo;
	Text_no        motd;

	sinfo = port->get_server_info();
	motd = sinfo->get_motd_of_lyskom();
	cout << "motd_of_lyskom = " << motd << endl;

	if (motd != 0) {
	    cout << "Message of the day:\n";
	    print_text(session.texts()[motd]);
	}
    }

    if (session.login(this_person, &pwd[0]) != st_ok)
    {
	cerr << "Login error.\n";
	exit(1);
    }

    unread_confs = new Unread_confs(&session, this_person);
    command_interpreter();
}
